//   ***********************************************************
//   *               XAircraft Pilot Lamp                      *
//   *    LED & Buzzer Device for use with MultiWiiCopter.     *
//   *             by Thomas Black / mr.rc-cam                 *
//   *                  June-19-2012                           *
//   *                  Made in USA :)                         *
//   ***********************************************************
//
// Need help using this code? Please see http://www.multiwii.com/forum/viewtopic.php?f=6&t=1794
//
// *********************************************************************************************************************
//
// These routines for XAircraft's Pilot Lamp LED/Buzzer module are preliminary functions that may
// change after additional reverse-engineering of the XAircraft communication protocol is performed.
//
// *********************************************************************************************************************

union pl_reg {
    struct {
        unsigned grn: 1;
        unsigned blu: 1;
        unsigned red: 1;
        unsigned bzr: 1;
    } ctrl;
    uint8_t pl_ctrl_all;
};

// *********************************************************************************************************************
// PilotLamp() is called with the PL_XX parameter (see below). In order to prevent unneccessary
// increases in IMU cycle time, the bit-bang freq control waveform is only generated if the device (LED or buzzer) 
// state update is different from the previous state.
// Available device parameters are:
//    PL_INIT { must be called once in main sketch setup() }
//    PL_GRN_ON, PL_GRN_OFF { Green LED control }
//    PL_BLU_ON, PL_BLU_OFF { Blue  LED control }
//    PL_RED_ON, PL_RED_OFF { Red   LED control }
//    PL_BZR_ON, PL_BZR_OFF { Buzzer control }
//
void PilotLamp(uint16_t device)
{
    uint8_t i;
    static union pl_reg mode;
    
    if(device == PL_INIT) {                            // Initialize the Pilot Lamp.
        mode.pl_ctrl_all = 0x00;                       // Reset all LED and Buzzer status states.
        for (i=0;i<4;i++) {                            // Create the required freq waveforms to control the various modes.
            PL_PIN_ON;
            delayMicroseconds(PL_GRN_OFF);
            PL_PIN_OFF;
            delayMicroseconds(PL_GRN_OFF);

            PL_PIN_ON;
            delayMicroseconds(PL_BLU_OFF);
            PL_PIN_OFF;
            delayMicroseconds(PL_BLU_OFF);

            PL_PIN_ON;
            delayMicroseconds(PL_RED_OFF);
            PL_PIN_OFF;
            delayMicroseconds(PL_RED_OFF);
              
            PL_PIN_ON;
            delayMicroseconds(PL_BZR_OFF);
            PL_PIN_OFF;
            delayMicroseconds(PL_BZR_OFF);              
        }
    }
    else {                                                  // Check to see if the new state request is different than the current state.
        if(device==PL_GRN_OFF && mode.ctrl.grn==1) {
            mode.ctrl.grn = 0;
        }
        else if(device==PL_GRN_ON && mode.ctrl.grn==0) {
            mode.ctrl.grn = 1;
        }
        else if(device==PL_BLU_OFF && mode.ctrl.blu==1) {
            mode.ctrl.blu = 0;
        }
        else if(device==PL_BLU_ON && mode.ctrl.blu==0) {
            mode.ctrl.blu = 1;
        }
        else if(device==PL_RED_OFF && mode.ctrl.red==1) {
            mode.ctrl.red = 0;
        }
        else if(device==PL_RED_ON && mode.ctrl.red==0) {
            mode.ctrl.red = 1;
        }
        else if(device==PL_BZR_OFF && mode.ctrl.bzr==1) {
            mode.ctrl.bzr = 0;
        }
        else if(device==PL_BZR_ON && mode.ctrl.bzr==0) {
            mode.ctrl.bzr = 1;
        }
        else {                                             // No state changes
            PL_PIN_OFF;
            return;                                       // Skip signal generation.
        }
        
        for (i=0;i<4;i++) {                                // Create the freq signal to activate Pilot Lamp.
            PL_PIN_ON;
            delayMicroseconds(device);
            PL_PIN_OFF;
            delayMicroseconds(device);
        }
    }
    
    return;
}
    

// *****************************************************************************************************************
// PilotLampTest() can be used to troubleshoot the LED/Buzzer module. Place it in the main loop and you 
// should see the three LED's sequence in order with a periodic short beep.
void PilotLampTest(void)
{
     static uint8_t cam1 = 0;
     static uint8_t cam2 = 0;
     static uint8_t state = 0;
     
     if(cam1++ > 40) {                                // Update rate is perhaps 2Hz, but will vary.
         cam1 = 0;
         if(cam2++ > 20 && state==1) {                // Periodically beep the buzzer (every few seconds).
             cam2 = 0;
             PilotLamp(PL_BZR_ON);
             delay(200);                              // Brute force delay is needed because maim loop will immediately turn off buzzer.
         }
         else {
             switch(state) {                          // Light-chase Cycle the LED's.
                 case 0:
                     state++;
                     PilotLamp(PL_BZR_OFF);
                     break;
                 case 1:
                     state++;
                     PilotLamp(PL_GRN_ON);
                     break;
                 case 2:
                     state++;
                     PilotLamp(PL_GRN_OFF);
                     PilotLamp(PL_BLU_ON);
                     break;
                 case 3:
                     state++;
                     PilotLamp(PL_BLU_OFF);
                     PilotLamp(PL_RED_ON);
                     break;
                 case 4:
                     state = 0;
                     PilotLamp(PL_RED_OFF);
                     break;
             }
        }             
     }
  
    return;
}

// *********************************************************************************************************************

